local skin = "sonic" // Skin to use

local HMRDROP_MAXSPEED = 12*FRACUNIT // Max Horizontal Speed
local HMRDROP_DOWNSPEED = -(30*FRACUNIT) // Minimum Speed going down
local HMRDROP_ENDSPEED = (6*FRACUNIT) // Speed going up after a hammer drop

freeslot("S_PLAY_HMRDROP", "S_PLAY_HMRDROP_SLAM", "S_PLAY_HMRDROP_END", "sfx_hmdr", "sfx_hmla", "sfx_hmdf")

states[S_PLAY_HMRDROP] = {
	sprite = SPR_PLAY;
	frame = SPR2_ROLL;
	tics = 1;
	action = nil;
	var1 = 0;
	var2 = 0;
	nextstate = S_PLAY_HMRDROP;
};

states[S_PLAY_HMRDROP_SLAM] = {
	sprite = SPR_PLAY;
	frame = SPR2_ROLL;
	tics = 4;
	action = nil;
	var1 = 0;
	var2 = 0;
	nextstate = S_PLAY_HMRDROP_END;
};

function A_HmrDropEnd(mo)
	local player = mo.player
	if mo and mo.valid then
		if player and player.valid then
			if player.powers[pw_justsprung] > 0 then
				P_SetObjectMomZ(mo, mo.momz * 125/100)
				return
			end
		end
		if mo.momz > 2*FRACUNIT then
			P_SetObjectMomZ(mo, 1*FRACUNIT)
		end
	end
end

states[S_PLAY_HMRDROP_END] = {
	sprite = SPR_PLAY;
	frame = SPR2_ROLL;
	tics = 1;
	action = A_HmrDropEnd;
	var1 = 0;
	var2 = 0;
	nextstate = S_PLAY_JUMP;
};

local function checkSkin(mo)
	if mo and mo.valid and mo.skin == skin then
		return true
	end
	return false
end

local function stopDrop(mo)
	local player = mo.player
	local hmrdrop = mo.hmrdrop
	hmrdrop.dropping = false
	hmrdrop.dropped = true
	hmrdrop.sprung = false
	P_RadiusAttack(mo, mo, 80*FRACUNIT)
	P_StartQuake(5*FRACUNIT, 5, {mo.x, mo.y, mo.z})
	S_StartSound(mo, sfx_hmla)
	if not (player.powers[pw_justsprung] > 0) then
		P_SetObjectMomZ(mo, HMRDROP_ENDSPEED)
		mo.state = S_PLAY_HMRDROP_SLAM
		player.pflags = $ & ~PF_NOJUMPDAMAGE
	end
	
	for i = 1, 30 do
		local dust = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_PARTICLE)
		dust.scale = mo.scale * P_RandomRange(1,2)
		dust.destscale = 0
		dust.scalespeed = FRACUNIT/P_RandomRange(6,8)
		dust.frame = $|TR_TRANS40
		dust.fuse = 16
		P_InstaThrust(dust, FixedAngle(P_RandomRange(0, 360)*FRACUNIT), FixedMul(mo.scale, P_RandomRange(4, 6)*FRACUNIT))
	end
end

addHook("ThinkFrame", function()
	for player in players.iterate do
		local mo = player.mo
		if mo and mo.valid and checkSkin(mo) then
			if mo.hmrdrop == nil then
				mo.hmrdrop = {}
				mo.hmrdrop.dropping = false
				mo.hmrdrop.dropped = false
				mo.hmrdrop.spiked = false
				mo.hmrdrop.spikedtimer = 0
			end
			local hmrdrop = mo.hmrdrop
			if not (player.playerstate == PST_LIVE and mo.state ~= S_PLAY_PAIN) and mo.hmrdrop.dropping then
				mo.hmrdrop.dropping = false
			end
			// Allows hammer dropping again
			if mo.hmrdrop.dropped then
				if P_IsObjectOnGround(mo) then
					mo.hmrdrop.dropped = false
				end
			end
			if hmrdrop.spiked and P_IsObjectOnGround(mo) then
				hmrdrop.spiked = false
			end
			hmrdrop.spikedtimer = max($ - 1, 0)
			
			// i dont really feel like doing this so i just copied it from the mighty port
			// thank you appleblurt, monster iestyn, emong, and tripel the fox
			local grounded = P_IsObjectOnGround(mo) or P_IsObjectInGoop(mo)
			if grounded then
				for x=player.mo.x,player.mo.x+player.mo.radius,player.mo.radius
					for y=player.mo.y,player.mo.y+player.mo.radius,player.mo.radius
						local ss = R_PointInSubsector(x,y)
						for fof in ss.sector.ffloors()
							if (fof.flags & FF_BUSTUP) and (fof.flags & FF_EXISTS)
								if (player.mo.z+player.mo.momz==fof.topheight or player.mo.z+player.mo.height+player.mo.momz==fof.bottomheight or (player.mo.eflags & MFE_VERTICALFLIP and (player.mo.z+player.mo.momz==fof.bottomheight or player.mo.z-player.mo.height-player.mo.momz==fof.topheight)))
									EV_CrumbleChain(ss.sector, fof)
									hmrdrop.dropping = true
									mo.eflags = $ & ~MFE_JUSTHITFLOOR
									grounded = false
								end
							end
						end
					end
				end
			end
			
			if mo.hmrdrop.dropping then
				// Aerial
				if not grounded then
					if mo.momz > HMRDROP_DOWNSPEED then
						P_SetObjectMomZ(mo, HMRDROP_DOWNSPEED)
					end
					mo.state = S_PLAY_HMRDROP
					if player.speed > HMRDROP_MAXSPEED then
						local angle = R_PointToAngle2(mo.x, mo.y, mo.x + mo.momx, mo.y + mo.momy)
						P_InstaThrust(mo, angle, HMRDROP_MAXSPEED)
					end
					player.pflags = $ & ~PF_NOJUMPDAMAGE
					local trail = P_SpawnGhostMobj(mo)
					trail.fuse = 10
				elseif grounded or (mo.eflags & MFE_JUSTHITFLOOR) then // Landing
					stopDrop(mo)
				end
			end
			if player.powers[pw_justsprung] ~= 0 and mo.state == S_PLAY_HMRDROP then
				stopDrop(mo)
			end
		end
	end
end)

addHook("JumpSpinSpecial", function(player)
	local mo = player.mo
	if not (mo and mo.valid and checkSkin(mo)) then
		return
	end
	if not (mo.state == S_PLAY_JUMP or mo.state == S_PLAY_ROLL) then
		return
	end
	
	local hmrdrop = mo.hmrdrop
	if hmrdrop and not hmrdrop.dropping and not hmrdrop.dropped then
		mo.hmrdrop.dropping = true
		player.pflags = $|PF_THOKKED
		player.pflags = $ & ~PF_NOJUMPDAMAGE
		S_StartSound(mo, sfx_hmdr)
	end
end)

addHook("PreThinkFrame", function(player)
	for player in players.iterate do
		local mo = player.mo
		if mo and mo.valid and checkSkin(mo) then
			local hmrdrop = mo.hmrdrop
			if hmrdrop then
				if hmrdrop.dropping and (player.cmd.buttons & BT_USE) then
					player.cmd.buttons = $ & ~BT_USE
				end
			end
		end
	end
end)

addHook("MobjDamage", function(target, inflictor, source, damage, damagetype)
	if target and target.valid and damagetype then
		if checkSkin(target) and target.hmrdrop and target.player and target.player.valid then
			local hmrdrop = target.hmrdrop
			if damagetype == DMG_SPIKE and hmrdrop.dropping then
				target.eflags = $|MFE_JUSTHITFLOOR
				return true
			end
			if damagetype == DMG_SPIKE and not hmrdrop.spiked then
				hmrdrop.spiked = true
				hmrdrop.spikedtimer = 10
				P_SetObjectMomZ(target, 7*FRACUNIT)
				if target.player.speed > 1*FRACUNIT then
					local angle = R_PointToAngle2(target.x, target.y, target.x + target.momx, target.y + target.momy)
					P_InstaThrust(target, angle * FixedAngle(180*FRACUNIT), 8*FRACUNIT)
				end
				if target.state == S_PLAY_ROLL or target.state == S_PLAY_JUMP then
					target.state = S_PLAY_FALL
					target.player.pflags = $|PF_NOJUMPDAMAGE
					S_StartSound(mo, sfx_hmdf)
				end
				return true
			elseif damagetype == DMG_SPIKE and hmrdrop.spikedtimer > 0 then
				return true
			end
		end
	end
end, MT_PLAYER)

addHook("ShouldDamage", function(target, inflictor, source, damage, damagetype)
	if target and target.valid and inflictor and inflictor.valid then
		if checkSkin(target) and target.hmrdrop and target.player and target.player.valid then
			local hmrdrop = target.hmrdrop
			if inflictor.type == MT_SPIKE then
				if hmrdrop.dropping then
					P_KillMobj(inflictor, target)
					S_StartSound(target, inflictor.info.deathsound)
					return false
				end
			end
		end
	end
end, MT_PLAYER)